/*
Copyright (c) 2009, 2010, 2011 Patrice Nadeau
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
    * Neither the name of Patrice Nadeau nor the
      names of its contributors may be used to endorse or promote products
      derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL Patrice Nadeau BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

/**
 * @internal
 * @file terminal.c
 * @brief Various functions for a terminal session
 * @details Terminal functions on an AVR uC
 * @author Patrice Nadeau
 */

#include "terminal/terminal.h"

/**
 * @brief Replace the non standard itoa function
 * @note Only works for unsigned 8 bit integer (0 @ 255)
 */
static char* _itoa(char *string, uint8_t nb) {
	char str1[4];
	uint8_t ctr1, ctr2, modulo;
	ctr1 = 0;												/* string empty */
	do {
		modulo = nb % 10;									/* the modulo will be the most right digit */
		nb = nb/10;
		str1[ctr1++] = 48 + modulo;							/* ASCII 48 + modulo */
	} while (nb > 0);
	str1[ctr1] = '\0';										/* finish the string */
	/* invert the string */
	ctr1 = strlen(str1);									/* find how many char ton invert */
	ctr2 = 0;
	do {
		string[ctr2++] = str1[--ctr1];						/* take the last char in str1 and put it first in string */
	} while (ctr1 != 0);
	string[ctr2] = '\0';									/* finish the string */
	return string;
}

void terminal_Position (uint8_t row, uint8_t column) {
   char temp[10] = "\33["; 									/* ESC sequence */
   char temp2[4];
   _itoa(temp2, row);										/* convert row to a string and append to the temp string */
   strcat (temp, temp2);
   strcat (temp, ";");
   _itoa (temp2, column);									/* convert column to a string and append to the temp string */
   strcat (temp, temp2);
   strcat (temp, "H");
   usart_SendString (temp);
}

void terminal_ReadStringMax (char *str, uint8_t max) {
   uint8_t c;
   uint8_t dummy;
   uint8_t x;
   x = 0;
   do {
      while (!usart_IsCharacter()) {						/* wait a character to be in the buffer */
      };
      c = usart_ReceiveCharacter ();
      if ((x < max) && (c > 31) && (c < 127)) {				/* check if we still have some place left and not a control character */
         *str++ = (char) c;
         x++;
         usart_SendCharacter (c);
      }
      else if (x != 0) { 									/* a control char has been used, must check if already at beginning */
         switch (c) { 										/* valid commands are BackSpace, ESC */
            case ascii_ESC :								/* ESC erase everything typed before, one character at the time */
               do {
                  dummy = *--str;							/* rewind  1 position in the string */
                  --x;
                  usart_SendString_P(terminal_BACKSPACE);	/* Backspace cursor on screen */
                  usart_SendCharacter (ascii_SPACE); 		/* erase the last character on screen */
                  usart_SendString_P(terminal_BACKSPACE); 	/* Backspace cursor on screen */
               } while (x > 0);
               break;
            case ascii_BS : 								/* Backspace rewind 1 position to the left */
               dummy= *--str;								/* backward 1 position into the string */
               --x;
               usart_SendString_P(terminal_BACKSPACE); 		/* Backspace cursor on screen */
               usart_SendCharacter(ascii_SPACE); 			/* erase the last character on screen */
               usart_SendString_P(terminal_BACKSPACE);		/* Backspace cursor on screen */
               break;
            }
          }
   } while (c != ascii_ENTER); 								/* enter has been pressed */
   *str = ascii_NULL; 										/* Terminate the string */
}

void terminal_PlaceString (uint8_t row, uint8_t col, const char *str, const char *foreground, const char *background, const char *attributes) {
	terminal_Position(row, col);
	usart_SendString(attributes);
	usart_SendString(background);
	usart_SendString(foreground);
	usart_SendString(str);
	usart_SendString(terminal_ATTR_RESET);					/* Reset the cursor */

}

void terminal_PlaceString_P (uint8_t row, uint8_t col, const char *str, const char *foreground, const char *background, const char *attributes) {
	terminal_Position(row, col);
	usart_SendString_P(attributes);
	usart_SendString_P(background);
	usart_SendString_P(foreground);
	usart_SendString_P(str);
	usart_SendString_P(terminal_ATTR_RESET);				/* Reset the cursor */

}

/* change log
 * 2008-12-26 Patrice Nadeau
 * First draft
 * 2010-08-01
 * In terminalPosition, replaced the call to the non standard itoa() with my own static _itoa()
 * 2010-08-07
 * In TerminalReadString, replaced the check isalnum() to include all characters except control ones
 * 2010-08-14
 * In terminalReadString(), changed for using the define's
 * 2010-10-03 Patrice Nadeau
 * Renamed every function for my new standard include_FunctionName()
 * 2010-10-26 Patrice Nadeau
 * Renamed terminalProcessStar to terminal_ProcessStar (was forgotten previously)
 * 2010-10-27 Patrice Nadeau
 * terminal_ReadString() : renamed ascii_XX to usart_XX, added a dummy variable to remove compiler warnings
 * 2010-10-30 Patrice Nadeau
 * terminal_SendString() : changed "char string[]" to "const char *data" (http://www.avrfreaks.net/index.php?name=PNphpBB2&file=viewtopic&t=38003&postdays=0&postorder=asc)
 * terminal_SendString_P() : added to use the PROGMEM attribute
 * terminal_PlaceString_P() : added to use the PROGMEM attribute
 * Moved to usart.c : SendString(), SendString_P(), Byte(), Long()
 * Renamed : terminalReadString() to terminal_ReadStringMax()
 * 2010-11-07 Patrice Nadeau
 * terminal_ReadStringMax() : added a loop with usart_IsCharacter() since usart_ReadCharcer() does not check the buffer anymore
 * 2011-01-06 Patrice Nadeau
 * terminal_ReadStringMax() : change to reflect the use of usart_CHAR_XX in lieu of usart_XX
 * 2011-01-08 Patrice Nadeau
 * Changed comments for Doxygen
 * 2011-03-06 Patrice Nadeau
 * terminal_ReadStringMax : changed to use the new define in ascii.h
 * 2011-03-27 Patrice Nadeau
 * terminal_ReadStringMax : replace a check with isalnum,
 * terminal_PlaceString_P : modified to use all progmem functions
 * 2011-04-14 Patrice Nadeau
 * terminal_PlaceString : added the const qualifier to the parameters
 */
