/**
 * @section License
Copyright (c) 2009, 2010, 2011 Patrice Nadeau
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
    * Neither the name of Patrice Nadeau nor the
      names of its contributors may be used to endorse or promote products
      derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL Patrice Nadeau BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

/**
 * @section Target
 * - ATmega with a USART port
 * - ATmega168 (Tested)
 * - ATmega8515 (Tested)
 * @section information More informations
 * Datasheet
*/

#ifndef TERMINAL_H_
#define TERMINAL_H_
/**
 * @file terminal.h
 * @brief Various functions for a terminal session with an AVR
 * @author Patrice Nadeau
 * @n email : patricen@telwarwick.net
 * @n www : http://nadeaup.homeip.net:8080/
 */

#include <ctype.h> 										/* for isalnum */
#include <string.h>
#include <avr/pgmspace.h> 								/* to use the PROGMEM attribute */

#include "usart/usart.h"

/**
 * @name Useful codes
 * @{*/
#define terminal_RESET "\033c"
#define terminal_ERASE_EOL "\033[K"
#define terminal_CLEARSCREEN "\033[2J"
#define terminal_BACKSPACE "\033[1D"
/** @} */

/**
 * @name Useful characters
 * @{
 * */
#define terminal_CR_LF "\r\n"
/** @} */


/**
 * @name Foreground color code
 * @{
 * */
#define terminal_FG_BLACK "\033[30m"
#define terminal_FG_RED "\033[31m"
#define terminal_FG_GREEN "\033[32m"
#define terminal_FG_YELLOW "\033[33m"
#define terminal_FG_BLUE "\033[34"
#define terminal_FG_MAGENTA "\033[35m"
#define terminal_FG_CYAN "\033[36m"
#define terminal_FG_WHITE "\033[37m"
/** @} */

/**
 * @name Background color code
 * @{
 * */
#define terminal_BG_BLACK "\033[40m"
#define terminal_BG_RED "\033[41m"
#define terminal_BG_GREEN "\033[42m"
#define terminal_BG_YELLOW "\033[43m"
#define terminal_BG_BLUE "\033[44m"
#define terminal_BG_MAGENTA "\033[45m"
#define terminal_BG_CYAN "\033[46m"
#define terminal_BG_WHITE "\033[47m"
/** @} */

/**
 * @name Attributes codes
 * @{
 * */
#define terminal_ATTR_RESET "\033[0m"
#define terminal_ATTR_BRIGHT "\033[1m"
/** @} */


/**
 * @brief Place the cursor at @a row @a column
 * @param[in] row Row
 * @param[in] column Column
 */
void terminal_Position (uint8_t row, uint8_t column);

/**
 * @brief Read @a max characters into @a *str
 * @param[in] max Maximum characters to read
 * @param[out] *str String
 * @note The following keys are permitted
 * - ESC : erase the current line
 * - Backspace : go back one cursor left
 * - Enter : terminate the string
 */
void terminal_ReadStringMax (char *str, uint8_t max);

/**
 * @brief Clear the terminal screen
 */
static inline void terminal_CLS (void) {
	usart_SendString_P(terminal_CLEARSCREEN);
}

/**
 * @brief Place @a *string at @a row @a col with @a foreground color, @a background color and @a attributes
 * @param[in] row Row on the screen
 * @param[in] col Column on the screen
 * @param[in] *str String to display
 * @param[in] *foreground Foreground color @n Possible values :
 * - @n terminal_FG_xx
 * @param[in] *background Background color @n Possible values :
 * - @n terminal_BG_xx
 * @param[in] *attributes Attributes of the string @n Possible values :
 * - @n terminal_ATTR_XX
 */
void terminal_PlaceString (uint8_t row, uint8_t col, const char *str, const char *foreground, const char *background, const char *attributes);

/**
 * @brief Place PROGMEM @a *string at @a row @a col with @a foreground / @a background color and @a attributes
 * @param[in] row Row on the screen
 * @param[in] col Column on the screen
 * @param[in] *str PROGMEM string to display
 * @param[in] *foreground Foreground color @n Possible values :
 * - @n terminal_FG_xx
 * @param[in] *background Background color @n Possible values :
 * - @n terminal_BG_xx
 * @param[in] *attributes Attributes of the string @n Possible values :
 * - @n terminal_ATTR_XX
 */
void terminal_PlaceString_P (uint8_t row, uint8_t col, const char *str, const char *foreground, const char *background, const char *attributes);

/**
 * @brief Reset the terminal screen
 */
static inline void terminal_Reset (void) {
	usart_SendString(terminal_RESET);
}

/**
 * @brief Show a star rotating
 * @note Mostly for show since no work is done while executing.
 */
static inline void terminal_ProcessStar(void){
	usart_SendCharacter('|');
	usart_SendString(terminal_BACKSPACE);
	usart_SendCharacter('/');
	usart_SendString(terminal_BACKSPACE);
	usart_SendCharacter('-');
	usart_SendString(terminal_BACKSPACE);
	usart_SendString("\\");
	usart_SendCharacter('-');
	usart_SendString(terminal_BACKSPACE);
}


#endif /* TERMINAL_H_ */


/* change log
 * 2009-07-27 Patrice Nadeau
 * First draft
 * 2010-08-01
 * Removed the #include for stblib.h, the non standard itoa function is no longuer used
 * 2010-10-27 Patrice Nadeau
 * Added #define terminal_CR_LF
 * 2010-10-30 Patrice Nadeau
 * terminal_SendString() : changed "char string[]" to "const char *data" (http://www.avrfreaks.net/index.php?name=PNphpBB2&file=viewtopic&t=38003&postdays=0&postorder=asc)
 * terminal_SendString_P() : added to use the PROGMEM attribute
 * terminal_PlaceString_P() : added to use the PROGMEM attribute
 * 2010-10-31 Patrice Nadeau
 * Moved to usart.h : SendString(), SendString_P(), Byte(), Long()
 * Renamed : terminalReadString() to terminal_ReadStringMax()
 * 2011-01-08 Patrice Nadeau
 * Changed comments for Doxygen
 * 2011-02-19 Patrice Nadeau
 * Added doxygen comments
 * 2011-04-14 Patrice Nadeau
 * terminal_PlaceString : added the const qualifier to the parameters
 */
